Exception: ActiveStash::CollectionDivergedError
- Inherits:
-
CollectionError
- Object
- StandardError
- ActiveStashError
- CollectionError
- ActiveStash::CollectionDivergedError
- Defined in:
- lib/active_stash/error.rb
Instance Method Summary collapse
Methods inherited from CollectionError
Constructor Details
This class inherits a constructor from ActiveStash::CollectionError
Instance Method Details
#default_message(name) ⇒ Object
45 46 47 48 49 50 |
# File 'lib/active_stash/error.rb', line 45 def (name) <<-STR The '#{name}' collection has diverged. Its settings are no longer reflected on the server. You probably want to recreate it and reindex your data. STR end |